##############################################################################
#
# main.tcl:
#       This file contains the procedures needed to create the toplevel window,
#       the control panel for the program. It also contains some procedures 
#       that don't fit in one of the other categories.
#
# MakeControl {}
#       Make the toplevel window, containing a list of folders, and a button 
#       menu, from which one can manipulate the folders, incorporate mail,
#       compose a message, and call up the alias window.
#
# AdaptFolMenus {}
#	Adapt folder menus to current values for FolTree
#	TODO: adapt more than the main window alone.
#
# DisplayFols {}
#       Put the contents of FolList in the folderwindow
#
# CheckForMail {}
#       Check the Maildrop for incoming mail.
#
# Loop {}
#       The loop used for checking for new mail and so
#
##############################################################################

proc makeControl {} {
    global BM ALIAS ADDRESS PREF FolList FolTree MH FONT

    wm withdraw .
    
    toplevel .tkmh -class Control
    wm title .tkmh TkMH
    wm iconbitmap .tkmh @$BM(nomail)

    scrollbar .tkmh.fols -command {.tkmh.foll yview}
    listbox .tkmh.foll -yscroll {.tkmh.fols set} -height 40

    frame .tkmh.abar
    checkbutton .tkmh.abar.archive -variable PREF(DispArchive) \
      -font $FONT(ProB) \
      -text "Display Archived Folders" \
      -relief raised \
      -command {BuildFolList 0; set FolTree [BuildFolTree $FolList];
       DisplayFols $FolList; AdaptFolMenus}

    frame .tkmh.b -border 1
    button .tkmh.b.quit -text Quit! -command {destroy .tkmh; exit}
    button .tkmh.b.inc -bitmap @$BM(nomail) -command {IncMail}
    button .tkmh.b.help -text Help -command {Help Control}
    menubutton .tkmh.b.conf -text Configure -menu .tkmh.b.conf.m
    menubutton .tkmh.b.f -text "Folder" -menu .tkmh.b.f.m

    menu .tkmh.b.f.m
    .tkmh.b.f.m add command -label Open -command {
      if {[set Sel [GetListboxSel .tkmh.foll]] != {}} {
        FlashSel .tkmh.foll 2; OpenFol $Sel}}
    .tkmh.b.f.m add command -label Remove -command {
      if {[set Sel [GetListboxSel .tkmh.foll]] != {}} {
        FlashSel .tkmh.foll 2; RemoveFol $Sel}}
    .tkmh.b.f.m add command -label Rename -command {
      if {[set Sel [GetListboxSel .tkmh.foll]] != {}} {
        FlashSel .tkmh.foll 2; RenameFol $Sel}}
    .tkmh.b.f.m add command -label Archive -command {
      if {[set Sel [GetListboxSel .tkmh.foll]] != {}} {
        FlashSel .tkmh.foll 2; ArchiveFol $Sel}}
    .tkmh.b.f.m add command -label UnArchive -command {
      if {[set Sel [GetListboxSel .tkmh.foll]] != {}} {
        FlashSel .tkmh.foll 2; UnArchiveFol $Sel}}
    .tkmh.b.f.m add separator
    .tkmh.b.f.m add command -label {Create New} -command \
      {if {[set Fol [Ask {Folder to Create?}]] != {}} {CreateFol $Fol}}
    .tkmh.b.f.m add separator
    .tkmh.b.f.m add command -label "Pick Folders" -command \
      {if {[set EXPR [Ask {Match against:}]] != {}} {
        set PickList {}
        foreach Fol $FolList {if [regexp $EXPR $Fol] {lappend PickList $Fol}}
        DisplayFols $PickList}}
    .tkmh.b.f.m add command -label {Redisplay Folders} -command {DisplayFols $FolList}
    .tkmh.b.f.m add command -label {Reread Folder Tree} -command \
      {BuildFolList 0; set FolTree [BuildFolTree $FolList]; 
       DisplayFols $FolList; AdaptFolMenus}
    .tkmh.b.f.m add separator
    .tkmh.b.f.m add command -label {Convert To Elm} -command {
      if {[set Sel [GetListboxSel .tkmh.foll]] != {}} { FlashSel .tkmh.foll 2; ConvToElm $Sel}}

    AdaptFolMenus

    menu .tkmh.b.conf.m
    .tkmh.b.conf.m add command -label "Aliases" -command {AliasWindow}
    .tkmh.b.conf.m add command -label TkMH -command ConfTkMH
    .tkmh.b.conf.m add command -label MH -command {
        if [Confirm "All Message, Edit and Folder windows will be closed.\nContinue?\n"] {
          Refresh; ConfMH }}

     pack append .tkmh.abar \
      .tkmh.abar.archive {top expand fill}

     pack append .tkmh.b \
      .tkmh.b.inc {top expand fill} \
      .tkmh.b.conf {top expand fill} \
      .tkmh.b.f {top expand fill} \
      .tkmh.b.help {top expand fill} \
      .tkmh.b.quit {top expand fill}

    pack append .tkmh \
      .tkmh.abar {top fillx} \
      .tkmh.foll {left expand fill} \
      .tkmh.fols {left filly} \
      .tkmh.b {left filly} 

    bind .tkmh.foll <Double-Button-1> {
      if {[set Sel [GetListboxSel .tkmh.foll]] != {}} {
        if {[string last .tar.gz $Sel] != -1} {
          FlashSel .tkmh.foll 2; UnArchiveFol $Sel
        } else {
          FlashSel .tkmh.foll 2; OpenFol $Sel
        }
      }
    }
    update
    AdaptAliMenus 
    DisplayFols $FolList
    .tkmh.foll configure -selectmode single
    .tkmh.b.inc configure -state disabled
}

proc AdaptFolMenus {} {
    global FolTree
    
    # Control Window 'Open', 'Remove' and 'Rename'
    # Folder and Show windows 'Copy to' and 'Move to'
    # Edit window 'Save to' 
    foreach w [winfo children .] {
        switch [winfo class $w] {
          Control {
          }
          Folder {
            KillMenu $w.b.move.m
            MakeMenu $w.b.move.m $FolTree 1 {} -command \
             "if \[conv_listbox_seq [WF $w] movemsgs\] {MoveMsg [WF $w] %EntryPath}"
            KillMenu $w.b.copy.m
            MakeMenu $w.b.copy.m $FolTree 1 {} -command \
             "if \[conv_listbox_seq [WF $w] copymsgs\] {CopyMsg [WF $w] %EntryPath}"
          }
          ShowMsg {
            KillMenu $w.b.move.m
            MakeMenu $w.b.move.m $FolTree 1 {} -command \
             "write_seq [lindex [WS $w] 0] movemsgs [lindex [WS $w] 1]
              MoveMsg [lindex [WS $w] 0] %EntryPath"
            KillMenu $w.b.copy.m
            MakeMenu $w.b.copy.m $FolTree 1 {} -command \
             "write_seq [lindex [WS $w] 0] copymsgs [lindex [WS $w] 1]
              CopyMsg [lindex [WS $w] 0] %EntryPath"
          }
          EditMsg {
            KillMenu $w.b.save.m
            MakeMenu $w.b.save.m $FolTree 1 {} -command \
             "set Fol [lindex [WE $w] 0]; set Msg [lindex [WE $w] 1]
              set NewMsg \[FindLastFile %EntryPath\]; incr NewMsg
              SaveText $w.t \[RF \$MH(Path)/%EntryPath/\$NewMsg\]
              if \[winfo exists \[FW %EntryPath\]\] {ScanFol %EntryPath 1}"
          }
        }
    }
}

proc DisplayFols {List} {
    set curYview [.tkmh.fols get]
    .tkmh.foll delete 0 end
    foreach Fol $List { .tkmh.foll insert end $Fol }
    if {[llength $curYview] == 4} {
        .tkmh.foll yview [lindex $curYview 2]
    } elseif {[llength $curYview] == 2} {
        .tkmh.foll yview moveto [lindex $curYview 0]
    }
}

proc Loop {} {
    global PREF
    
    CheckForMail
    if {![winfo exists .tkmh]} {exit}
    after [expr $PREF(Delay) * 1000] Loop
}

set MDSize 0
set MDOldSize 0

proc CheckForMail {} {
    global MH BM PREF
    global MDSize MDOldSize

    if {![file isfile $MH(MailDrop)]} {
        return
    }

    set MDOldSize $MDSize
    set MDSize [file size $MH(MailDrop)]

    if {$MDSize > $MDOldSize} {
        # There is new mail
        if $PREF(DeIcon) {wm deiconify .tkmh; raise .tkmh}
        .tkmh.b.inc configure -bitmap @$BM(mail) -state active
        wm iconbitmap .tkmh @$BM(mail)
        .tkmh.b.inc flash; .tkmh.b.inc flash
    }

    if {$MDSize < $MDOldSize && $MDSize <= 1} {
        # The maildrop file must be emptied by something else
        .tkmh.b.inc configure -bitmap @$BM(nomail) -state disabled
        wm iconbitmap .tkmh @$BM(nomail)
    }
}

proc IncMail {} {
    global MH BM PREF

    # the check is normally not needed, but just in case something 
    # outside TkMH is working at the maildrop at the same time, or
    # the maildrop is not accessible for some reason
    if {[catch {exec $MH(bindir)/inc +$MH(Inbox)}] == 1} {
        MsgWindow "Something wrong with the Maildrop file!\
                   \nMaybe it's locked because you were just receiving mail.\
                   \n\n Try again.\n"
        return
    }

    .tkmh.b.inc configure -bitmap @$BM(nomail) -state disabled
    wm iconbitmap .tkmh @$BM(nomail)
    if $PREF(OpenInbox) {
        OpenFol $MH(Inbox) 1
    }
}
